@@ -10,6 +10,7 @@
 #include <stdarg.h>
 #include <setjmp.h>
 #include <stdio.h>
+#include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <netinet/in.h>
@@ -17,6 +18,7 @@
 #include <pwd.h>
 #include <stdbool.h>
 #include <math.h>
+#include <netinet/tcp.h>
 
 #include "config.h"
 #if defined (HAVE_PATH_H)
@@ -45,6 +47,10 @@
 
 #include "osso-gps.h"
 
+#ifdef USE_DOMAIN_SOCKET_IPC
+static const char const *path = GPSD_SERVER_CLIENT_PATH;
+#endif
+
 
 /* We only need one function from gpsctrl.c and it is not mentioned
  * in the header file (made like that in purpose).
@@ -83,7 +89,6 @@ static int client_count = 0;  /* Fixes: NB#62022 */
 /* Where to find the list of DGPS correction servers, if there is one */
 #define DGPS_SERVER_LIST	"/usr/share/gpsd/dgpsip-servers"
 
-
 /*
  * The name of a tty device from which to pick up whatever the local
  * owning group for tty devices is.  Used when we drop privileges.
@@ -282,13 +287,16 @@ static int filesock(char *filename)
     int sock;
 
     /*@ -mayaliasunique @*/
-    if ((sock = socket(AF_UNIX, SOCK_STREAM, 0)) < 0) {
+    if ((sock = socket(AF_LOCAL, SOCK_STREAM, 0)) < 0) {
 	gpsd_report(0, "Can't create device-control socket\n");
 	return -1;
     }
-    (void)strcpy(addr.sun_path, filename);
-    /*@i1@*/addr.sun_family = AF_UNIX;
-    (void)bind(sock, (struct sockaddr *) &addr,  (int)sizeof(addr));
+    (void)strncpy(addr.sun_path, filename, sizeof(addr.sun_path)-1);
+    /*@i1@*/addr.sun_family = AF_LOCAL;
+    if (bind(sock, (struct sockaddr *) &addr,  (int)sizeof(addr))<0) {
+	    gpsd_report(0, "cannot bind %s\n", filename);
+	    return -1;
+    }
     if (listen(sock, QLEN) < 0) {
 	gpsd_report(0, "can't listen on local socket %s\n", filename);
 	return -1;
@@ -439,8 +447,52 @@ static int check_if_disabled(char *device)
 	return 0;
 }
 
+static void dump_mask(gps_mask_t mask, char *out, int outlen)
+{
+	int idx = 0;
 
+#define CHECK_MASK(value)						\
+	if (mask & value) {						\
+		idx += snprintf(out+idx, outlen-idx, "%s, ", #value);	\
+	}
 
+	CHECK_MASK(ONLINE_SET);
+	CHECK_MASK(TIME_SET);
+	CHECK_MASK(TIMERR_SET);
+	CHECK_MASK(LATLON_SET);
+	CHECK_MASK(ALTITUDE_SET);
+	CHECK_MASK(SPEED_SET);
+	CHECK_MASK(TRACK_SET);
+	CHECK_MASK(CLIMB_SET);
+	CHECK_MASK(STATUS_SET);
+	CHECK_MASK(MODE_SET);
+	CHECK_MASK(HDOP_SET);
+	CHECK_MASK(VDOP_SET);
+	CHECK_MASK(PDOP_SET);
+	CHECK_MASK(TDOP_SET);
+	CHECK_MASK(GDOP_SET);
+	CHECK_MASK(DOP_SET);
+	CHECK_MASK(HERR_SET);
+	CHECK_MASK(VERR_SET);
+	CHECK_MASK(PERR_SET);
+	CHECK_MASK(SATELLITE_SET);
+	CHECK_MASK(PSEUDORANGE_SET);
+	CHECK_MASK(USED_SET);
+	CHECK_MASK(SPEEDERR_SET);
+	CHECK_MASK(TRACKERR_SET);
+	CHECK_MASK(CLIMBERR_SET);
+	CHECK_MASK(DEVICE_SET);
+	CHECK_MASK(DEVICELIST_SET);
+	CHECK_MASK(DEVICEID_SET);
+	CHECK_MASK(ERROR_SET);
+	CHECK_MASK(CYCLE_START_SET);
+	CHECK_MASK(RTCM_SET);
+	CHECK_MASK(FIX_SET);
+
+	if (idx>1) {
+		out[idx-2]='\0';
+	}
+}
 
 static void detach_client(int cfd)
 {
@@ -481,7 +533,7 @@ static ssize_t throttled_write(int cfd, char *buf, ssize_t len)
 	return status;
     if (errno == EBADF)
 	gpsd_report(3, "client(%d) has vanished.\n", cfd);
-    else if (errno == EWOULDBLOCK && timestamp() - subscribers[cfd].active > NOREAD_TIMEOUT)
+    else if (errno == EWOULDBLOCK && (timestamp() - subscribers[cfd].active) > NOREAD_TIMEOUT)
 	gpsd_report(3, "client(%d) timed out.\n", cfd);
     else
 	gpsd_report(3, "client(%d) write: %s\n", cfd, strerror(errno));
@@ -1208,6 +1260,9 @@ static int handle_gpsd_request(int cfd, char *buf, int buflen)
 			whoami->device->gpsdata.d_decode_time - whoami->device->gpsdata.d_xmit_time,
 			whoami->device->poll_times[cfd] - whoami->device->gpsdata.d_xmit_time,
 			timestamp() - whoami->device->gpsdata.d_xmit_time);
+#ifdef DEBUG_PROFILE
+//	    gpsd_report(0, "profile send: \"%s\"\n", phrase);
+#endif
 	    break;
 	case '\r': case '\n':
 	    goto breakout;
@@ -1300,7 +1355,11 @@ int main(int argc, char *argv[])
     struct gps_device_t *device, *channel;
     struct sockaddr_in fsin;
     fd_set rfds, control_fds;
-    int i, option, msock, cfd, dfd; 
+    int i, option, msock, cfd, dfd;
+#ifdef USE_DOMAIN_SOCKET_IPC
+    int msock2;
+    struct sockaddr_un un_addr;
+#endif
     bool go_background = true;
     struct timeval tv;
     // extern char *optarg;
@@ -1309,6 +1368,7 @@ int main(int argc, char *argv[])
 #endif /* RTCM104_ENABLE */
     int was_active = 0;
     int timeout = DEFAULT_TIMEOUT;
+    gps_mask_t changed_summary;
 
     debuglevel = 0;
     while ((option = getopt(argc, argv, "F:D:S:d:fhNnpP:Vt:"
@@ -1412,6 +1472,19 @@ int main(int argc, char *argv[])
 	exit(2);
     }
     gpsd_report(1, "listening on port %s\n", gpsd_service);
+
+#ifdef USE_DOMAIN_SOCKET_IPC
+    unlink(path);
+    if ((msock2 = filesock(path)) < 0) {
+	gpsd_report(0,"command socket create failed, error %d\n",msock2);
+	exit(2);
+    }
+    if (chmod(path, S_IRUSR | S_IWUSR)<0) {
+	    gpsd_report(0, "Cannot set the permissions of %s [%s, %d]\n", path, strerror(errno), errno);
+    }
+    gpsd_report(1, "listening on %s\n", path);
+#endif
+
 #ifdef RTCM104_SERVICE
     /*@ -observertrans @*/
     if (!rtcm_service)
@@ -1433,9 +1506,12 @@ int main(int argc, char *argv[])
     }
 
 #ifdef NTPSHM_ENABLE
-    if (getuid() == 0) {
-	(void)nice(-10);		/* for precise timekeeping increase priority */
-	(void)ntpshm_init(&context, nowait);
+    if (geteuid() == 0) {
+#if 0
+	    // no way!!!
+	    (void)nice(-10);	/* for precise timekeeping increase priority */
+#endif
+	    (void)ntpshm_init(&context, nowait);
     }
 #endif /* NTPSHM_ENABLE */
 
@@ -1537,6 +1613,11 @@ int main(int argc, char *argv[])
     FD_SET(msock, &all_fds);
     gpsd_report (4, "adding fd %d to select\n", msock);
 
+#ifdef USE_DOMAIN_SOCKET_IPC
+    FD_SET(msock2, &all_fds);
+    gpsd_report (4, "adding unix fd %d to select\n", msock2);
+#endif
+
 #ifdef RTCM104_SERVICE
     FD_SET(nsock, &all_fds);
     gpsd_report (4, "adding rtcm104 fd %d to select\n", nsock);
@@ -1601,9 +1682,26 @@ int main(int argc, char *argv[])
 		gpsd_report(0, "accept: %s\n", strerror(errno));
 	    else {
 		int opts = fcntl(ssock, F_GETFL);
+		int optval = 1;
 
 		if (opts >= 0)
 		    (void)fcntl(ssock, F_SETFL, opts | O_NONBLOCK);
+
+#ifdef DISABLE_NAGLE
+		/* No Nagle, it just delays things unnecessarily */
+		if (setsockopt(ssock,
+			       IPPROTO_TCP,
+			       TCP_NODELAY,
+			       (const char *)&optval,
+			       sizeof(int))<0) {
+			gpsd_report(0,
+				    "setsockopt(TCP_NODELAY, accept #%d) failed (%d/%s)",
+				    ssock,
+				    errno,
+				    strerror(errno));
+		}
+#endif
+
 		gpsd_report(3, "client connect on %d\n", ssock);
 		FD_SET(ssock, &all_fds);
 		subscribers[ssock].active = timestamp();
@@ -1615,6 +1713,33 @@ int main(int argc, char *argv[])
 	    gpsd_report(4, "master sock %d cleared\n", msock);
 	}
 
+#ifdef USE_DOMAIN_SOCKET_IPC
+	/* always be open to new client connections */
+	if (FD_ISSET(msock2, &rfds)) {
+	    socklen_t alen = (socklen_t)sizeof(un_addr);
+	    /*@i1@*/int ssock = accept(msock2, (struct sockaddr *) &un_addr, &alen);
+
+	    if (ssock < 0)
+		gpsd_report(0, "accept unix: %s\n", strerror(errno));
+	    else {
+		int opts = fcntl(ssock, F_GETFL);
+		int optval = 1;
+
+		if (opts >= 0)
+		    (void)fcntl(ssock, F_SETFL, opts | O_NONBLOCK);
+
+		gpsd_report(3, "client connect on unixds %d\n", ssock);
+		FD_SET(ssock, &all_fds);
+		subscribers[ssock].active = timestamp();
+		subscribers[ssock].tied = false;
+		subscribers[ssock].requires = ANY;
+		client_count++;
+	    }
+	    FD_CLR(msock2, &rfds);
+	    gpsd_report(4, "master unix sock %d cleared\n", msock2);
+	}
+#endif
+
 #ifdef RTCM104_SERVICE
 	/* also to RTCM client connections */
 	if (FD_ISSET(nsock, &rfds)) {
@@ -1697,53 +1822,145 @@ int main(int argc, char *argv[])
 	    }
 
 	    /* get data from the device */
-	    changed = 0;
-	    if (channel->gpsdata.gps_fd >= 0 && FD_ISSET(channel->gpsdata.gps_fd, &rfds))
-	    {
-		gpsd_report(5, "polling %d\n", channel->gpsdata.gps_fd);
-		changed = gpsd_poll(channel);
-		if (changed == ERROR_SET) {
-			gpsd_report(3, "packet sniffer failed to sync up (%d)\n", channel->gpsdata.gps_fd);
-		    FD_CLR(channel->gpsdata.gps_fd, &all_fds);
-		    gpsd_deactivate(channel);
-		} 
-		if ((changed & ONLINE_SET) == 0) {
-                    /*
-		      gpsd_report(3, "GPS is offline (%lf sec since data)\n", 
-		      timestamp() - channel->gpsdata.online);
-		    */
-
-#if DBUS_ENABLE
-		    /* Send online status message when connection is closed.
-		     * Related to NB#60832
-		     */
-		    send_dbus_status_changed(channel, 0);
+	    changed = changed_summary = 0;
+	    if (channel->gpsdata.gps_fd >= 0 && FD_ISSET(channel->gpsdata.gps_fd, &rfds)) {
+		int remaining = 0;
+//#define DEBUG_MASK
+#ifdef DEBUG_MASK
+#define OUT_LEN 1024
+		char out[OUT_LEN+1];
 #endif
 
-		    FD_CLR(channel->gpsdata.gps_fd, &all_fds);
-		    gpsd_report(4, "gps (%d) offline, socket closed\n", channel->gpsdata.gps_fd);
-		    gpsd_deactivate(channel);
-		    notify_watchers(channel, "GPSD,X=0\r\n");
-		}
+		gpsd_report(5, "polling %d\n", channel->gpsdata.gps_fd);
+		do {
+			changed = gpsd_poll(channel);
+
+			/* Read all the data from internal buffer, not just the
+			 * first sentence!!! This way we get the data from internal buffer
+			 * to clients more than once/cycle. Fixes: NB#74463
+			 * Without this correction, the GPS fixes seem to come 1-2 seconds
+			 * too late. This only works for NMEA data, all the other protocols
+			 * are handled as before.
+			 */
+#ifdef DEBUG_MASK
+			dump_mask(changed, out, OUT_LEN);
+			gpsd_report(0, "%f: changed = 0x%08x, %s\n", timestamp(), changed, out);
+#endif
+
+			if (changed == ERROR_SET) {
+				gpsd_report(3, "packet sniffer failed to sync up (%d)\n",
+					    channel->gpsdata.gps_fd);
+				FD_CLR(channel->gpsdata.gps_fd, &all_fds);
+				gpsd_deactivate(channel);
+				break;
+			}
+
+
+			if ((changed & ONLINE_SET) == 0) {
+				/*
+				  gpsd_report(3, "GPS is offline (%lf sec since data)\n", 
+				  timestamp() - channel->gpsdata.online);
+				*/
+
+#if DBUS_ENABLE
+				/* Send online status message when connection is closed.
+				 * Related to NB#60832
+				 */
+				send_dbus_status_changed(channel, 0);
+#endif
+
+				if (channel->gpsdata.gps_fd >= 0) {  /* Fixes: NB#73205 */
+					gpsd_report(4, "gps (%d) offline, connection closed\n", channel->gpsdata.gps_fd);
+					FD_CLR(channel->gpsdata.gps_fd, &all_fds);
+					gpsd_deactivate(channel);
+				}
+				notify_watchers(channel, "GPSD,X=0\r\n");
+				break;
+			}
+
 #ifdef RTCM104_ENABLE
-		/* copy each RTCM-104 correction to all GPSes */
-		if ((changed & RTCM_SET) != 0) {
-		    for (gps = channels; gps < channels + MAXDEVICES; gps++)
-			if (gps->device_type != NULL && gps->device_type->rtcm_writer != NULL)
-			    (void)gps->device_type->rtcm_writer(gps, (char *)channel->outbuffer, channel->outbuflen);
-		}
+			/* copy each RTCM-104 correction to all GPSes */
+			if ((changed & RTCM_SET) != 0) {
+				for (gps = channels; gps < channels + MAXDEVICES; gps++)
+					if (gps->device_type != NULL && gps->device_type->rtcm_writer != NULL)
+						(void)gps->device_type->rtcm_writer(gps, (char *)channel->outbuffer, channel->outbuflen);
+			}
 #endif /* RTCM104_ENABLE */
+
+
+#if DBUS_ENABLE
+			if (changed &~ ONLINE_SET) {
+				if (changed & (LATLON_SET | MODE_SET)) 
+					if (channel && have_fix(channel)) {
+						send_dbus_fix (channel);
+					}
+			}
+
+			/* Check if we have received any "get status" messages.
+			 * This is probably better called before status_changed() so that
+			 * caller gets signals for its call and not the possible signals
+			 * sent by status_changed()
+			 */
+			check_dbus_messages(channels, MAXDEVICES);
+
+			if (changed) {
+				/* Send status message when something was changed */
+				send_dbus_status_changed(channel, 0);
+			}
+#endif
+
+			/* Remember the current position so that it can be saved when the gpsd
+			 * quits. This last position can then be viewed from control panel
+			 * applet.
+			 */
+			if (changed &~ ONLINE_SET) {
+				if (changed & (LATLON_SET | MODE_SET)) {
+					if (channel && have_fix(channel)) {
+						struct gps_data_t *gpsdata;
+
+						gpsdata = &(channel->gpsdata);
+						memcpy(&last_report.gpsdata, gpsdata, sizeof(struct gps_data_t));
+						last_report.set = channel->set;
+						//strncpy(last_report.buf, buf, strlen(buf)); 
+						last_report_status = 1; /* not empty */
+						gpsd_report(3,
+							    "GPS last report saved, "
+							    "lat=%f(%d), lon=%f, "
+							    "mode=%d(%d)\n",
+							    gpsdata->fix.latitude,
+							    (changed & LATLON_SET) == LATLON_SET,
+							    gpsdata->fix.longitude, 
+							    gpsdata->fix.mode,
+							    ((changed & MODE_SET) == MODE_SET));
+					}
+				}
+			}
+
+#ifdef DEBUG_MASK
+			dump_mask(changed, out, OUT_LEN);
+			gpsd_report(0, "%f: combined = 0x%08x, %s\n", timestamp(), changed, out);
+#endif
+
+			changed_summary |= changed;
+			remaining = channel->inbuflen - (channel->inbufptr - channel->inbuffer);
+		} while (remaining>0 && channel->outbuflen>0 && channel->packet_state==NMEA_RECOGNIZED);
+
+#ifdef DEBUG_MASK
+		dump_mask(changed, out, OUT_LEN);
+		gpsd_report(0, "%f: final = 0x%08x, %s\n", timestamp(), changed, out);
+#endif
 	    }
 
+
 	    for (cfd = 0; cfd < MAXSUBSCRIBERS; cfd++) {
 		/* some listeners may be in watcher mode */
 		if (subscribers[cfd].watcher) {
 		    char cmds[4] = ""; 
 		    channel->poll_times[cfd] = timestamp();
-		    if (changed &~ ONLINE_SET) {
-			if (changed & (LATLON_SET | MODE_SET))
+		    if (changed_summary &~ ONLINE_SET) {
+			if (changed_summary & (LATLON_SET | MODE_SET))
 			    (void)strcat(cmds, "o");
-			if (changed & SATELLITE_SET)
+			if (changed_summary & SATELLITE_SET)
 			    (void)strcat(cmds, "y");
 			if (channel->gpsdata.profiling!=0)
 			    (void)strcat(cmds, "$");
@@ -1752,53 +1969,6 @@ int main(int argc, char *argv[])
 			(void)handle_gpsd_request(cfd, cmds, (int)strlen(cmds));
 		}
 	    }
-
-#if DBUS_ENABLE
-	    if (changed &~ ONLINE_SET) {
-		    if (changed & (LATLON_SET | MODE_SET)) 
-			    if (channel && have_fix(channel))
-				    send_dbus_fix (channel);
-	    }
-
-	    /* Check if we have received any status get messages.
-	     * This is probably better called before status_changed() so that
-	     * caller gets signals for its call and not the possible signals
-	     * sent by status_changed()
-	     */
-	    check_dbus_messages(channels, MAXDEVICES);
-
-	    if (changed) {
-		    /* Send status message when something was changed */
-		    send_dbus_status_changed(channel, 0);
-	    }
-#endif
-
-	    /* Remember the current position so that it can be saved when the gpsd
-	     * quits
-	     */
-	    if (changed &~ ONLINE_SET) {
-		    if (changed & (LATLON_SET | MODE_SET)) {
-			    if (channel && have_fix(channel)) {
-				    struct gps_data_t*	gpsdata;
-
-				    gpsdata = &(channel->gpsdata);
-				    memcpy(&last_report.gpsdata, gpsdata, sizeof(struct gps_data_t));
-				    last_report.set = channel->set;
-				    //strncpy(last_report.buf, buf, strlen(buf)); 
-				    last_report_status = 1; /* not empty */
-				    gpsd_report(3,
-						"GPS last report saved, "
-						"lat=%f(%d), lon=%f, "
-						"mode=%d(%d)\n",
-						gpsdata->fix.latitude,
-						(changed & LATLON_SET) == LATLON_SET,
-						gpsdata->fix.longitude, 
-						gpsdata->fix.mode,
-						((changed & MODE_SET) == MODE_SET));
-			    }
-		    }
-	    }
-
 	}
 
 #ifdef NOT_FIXED
