@@ -9,6 +9,8 @@
 #include "config.h"
 #include "gps.h"
 
+#define GPSD_SERVER_CLIENT_PATH "/var/lib/gps/.gpsd_ipc"
+
 /* Some internal capabilities depend on which drivers we're compiling. */
 #ifdef EARTHMATE_ENABLE
 #define ZODIAC_ENABLE	
@@ -271,6 +273,7 @@ extern void gpsd_zero_satellites(/*@out@*/struct gps_data_t *sp)/*@modifies sp@*
 extern void gpsd_interpret_subframe(struct gps_device_t *, unsigned int[]);
 extern /*@ observer @*/ char *gpsd_hexdump(const void *, size_t);
 extern int netlib_connectsock(const char *, const char *, const char *);
+extern int netlib_connectsock_domain(const char *path);
 
 extern int ntpshm_init(struct gps_context_t *, bool);
 extern int ntpshm_alloc(struct gps_context_t *);
@@ -346,4 +349,9 @@ extern float roundf(float x);
 #if defined(__CYGWIN__)
 void cfmakeraw(struct termios *);
 #endif /* defined(__CYGWIN__) */
+
+enum {
+#include "packet_states.h"  // information is needed in gpsd.c
+};
+
 #endif /* _gpsd_h_ */
