@@ -385,7 +385,10 @@ static gps_mask_t processGPGSV(int count, char *field[], struct gps_device_t *se
 
     for (fldnum = 4; fldnum < count; ) {
 	if (session->gpsdata.satellites >= session->device_type->channels) {
-	    gpsd_report(0, "internal error - too many satellites!\n");
+#if 0
+		/* Do not print this, seems to happen all the time */
+		gpsd_report(0, "internal error - too many satellites!\n");
+#endif
 	    gpsd_zero_satellites(&session->gpsdata);
 	    break;
 	}
@@ -402,9 +405,13 @@ static gps_mask_t processGPGSV(int count, char *field[], struct gps_device_t *se
 	if (session->gpsdata.PRN[session->gpsdata.satellites] != 0)
 	    session->gpsdata.satellites++;
     }
+
+#if 0
+    /* Do not print this, seems to happen all the time. Happens always if "too many satellites" error is printed */
     if (session->driver.nmea.part == session->driver.nmea.await && atoi(field[3]) != session->gpsdata.satellites)
 	gpsd_report(0, "GPGSV field 3 value of %d != actual count %d\n",
 		    atoi(field[3]), session->gpsdata.satellites);
+#endif
 
     /* not valid data until we've seen a complete set of parts */
     if (session->driver.nmea.part < session->driver.nmea.await) {
